--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public schema';


SET search_path = public, pg_catalog;

--
-- Name: plpgsql_call_handler(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


ALTER FUNCTION public.plpgsql_call_handler() OWNER TO postgres;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: art_reference; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE art_reference (
    id integer NOT NULL,
    name_old character varying(256),
    description text,
    snapshot text,
    category character varying(256),
    s_status character varying(30),
    keywords text,
    "timestamp" timestamp without time zone DEFAULT now()
);


ALTER TABLE public.art_reference OWNER TO postgres;

--
-- Name: art_reference_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE art_reference_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE public.art_reference_id_seq OWNER TO postgres;

--
-- Name: art_reference_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE art_reference_id_seq OWNED BY art_reference.id;

--
-- Name: asset; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE asset (
    id serial NOT NULL,
    code character varying(30) NOT NULL,
    name character varying(100) NOT NULL,
    asset_type character varying(30),
    description text,
    discussion_bak text,
    "timestamp" timestamp without time zone DEFAULT now(),
    snapshot text,
    status text,
    director_notes text,
    episode_code character varying(30),
    asset_library character varying(100),
    pipeline_code character varying(30),
    s_status character varying(30)
);


ALTER TABLE public.asset OWNER TO postgres;

--
-- Name: asset_library; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE asset_library (
    id serial NOT NULL,
    code character varying(30) NOT NULL,
    title character varying(100),
    description text,
    padding smallint,
    "type" character varying(20)
);


ALTER TABLE public.asset_library OWNER TO postgres;

--
-- Name: asset_type; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE asset_type (
    id serial NOT NULL,
    code character varying(30) NOT NULL,
    description character varying(100)
);


ALTER TABLE public.asset_type OWNER TO postgres;

--
-- Name: bin; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE bin (
    id serial NOT NULL,
    code character varying(256),
    description text,
    "type" character varying(100),
    s_status character varying(30),
    label character varying(100)
);


ALTER TABLE public.bin OWNER TO postgres;

--
-- Name: composite; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE composite (
    id serial NOT NULL,
    name character varying(100),
    description text,
    shot_code character varying(100),
    snapshot text,
    "timestamp" timestamp without time zone DEFAULT now()
);


ALTER TABLE public.composite OWNER TO postgres;

--
-- Name: custom_property; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 

CREATE TABLE custom_property (
    id serial NOT NULL,
    search_type varchar(256),
    name varchar(256),
    description text,
    login varchar(256),
    PRIMARY KEY (id)
);


--
-- Name: episode; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE episode (
    id serial NOT NULL,
    code character varying(30) NOT NULL,
    description text,
    sort_order integer,
    "timestamp" timestamp without time zone DEFAULT now(),
    s_status character varying(30)
);


ALTER TABLE public.episode OWNER TO postgres;

--
-- Name: episode_instance; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE episode_instance (
    id serial NOT NULL,
    episode_code character varying(30) NOT NULL,
    asset_code character varying(100) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT now(),
    status text,
    "type" character varying(30)
);


ALTER TABLE public.episode_instance OWNER TO postgres;

--
-- Name: final_wave; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE final_wave (
    id serial NOT NULL,
    title character varying(30),
    episode_code character varying(30),
    line integer
);


ALTER TABLE public.final_wave OWNER TO postgres;

--
-- Name: funpack; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE funpack (
    id serial NOT NULL,
    episode_code character varying(30) NOT NULL,
    description text,
    status text,
    discussion text,
    files text,
    "timestamp" timestamp without time zone DEFAULT now()
);


ALTER TABLE public.funpack OWNER TO postgres;

--
-- Name: instance; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE instance (
    id serial NOT NULL,
    shot_code character varying(30) NOT NULL,
    asset_code character varying(100) NOT NULL,
    "type" character varying(30),
    "timestamp" timestamp without time zone DEFAULT now(),
    status text,
    name character varying(100)
);


ALTER TABLE public.instance OWNER TO postgres;

--
-- Name: layer; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE layer (
    id serial NOT NULL,
    name character varying(100),
    description text,
    shot_code character varying(100),
    snapshot text,
    "timestamp" timestamp without time zone DEFAULT now(),
    s_status character varying(30),
    "order" integer
);


ALTER TABLE public.layer OWNER TO postgres;

--
-- Name: layer_instance; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE layer_instance (
    id serial NOT NULL,
    asset_code character varying(100) NOT NULL,
    layer_id character varying(100) NOT NULL,
    "type" character varying(30),
    "timestamp" timestamp without time zone DEFAULT now(),
    status text,
    name character varying(100)
);


ALTER TABLE public.layer_instance OWNER TO postgres;

--
-- Name: leica; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE leica (
    id serial NOT NULL,
    snapshot text,
    episode_code character varying(30) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT now(),
    shot_code character varying(30),
    description text,
    code character varying(256)
);


ALTER TABLE public.leica OWNER TO postgres;

--
-- Name: naming; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE naming (
    id serial NOT NULL,
    search_type character varying(28),
    dir_naming text,
    file_naming text
);


ALTER TABLE public.naming OWNER TO postgres;

--
-- Name: nat_pause; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE nat_pause (
    id serial NOT NULL,
    title character varying(30),
    episode_code character varying(30)
);


ALTER TABLE public.nat_pause OWNER TO postgres;

--
-- Name: pga_forms; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_forms (
    formname character varying(64),
    formsource text
);


ALTER TABLE public.pga_forms OWNER TO postgres;

--
-- Name: pga_layout; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_layout (
    tablename character varying(64),
    nrcols smallint,
    colnames text,
    colwidth text
);


ALTER TABLE public.pga_layout OWNER TO postgres;

--
-- Name: pga_queries; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_queries (
    queryname character varying(64),
    querytype character(1),
    querycommand text,
    querytables text,
    querylinks text,
    queryresults text,
    querycomments text
);


ALTER TABLE public.pga_queries OWNER TO postgres;

--
-- Name: pga_reports; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_reports (
    reportname character varying(64),
    reportsource text,
    reportbody text,
    reportprocs text,
    reportoptions text
);


ALTER TABLE public.pga_reports OWNER TO postgres;

--
-- Name: pga_schema; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_schema (
    schemaname character varying(64),
    schematables text,
    schemalinks text
);


ALTER TABLE public.pga_schema OWNER TO postgres;

--
-- Name: pga_scripts; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pga_scripts (
    scriptname character varying(64),
    scriptsource text
);


ALTER TABLE public.pga_scripts OWNER TO postgres;

--
-- Name: pickup_request; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE pickup_request (
    id serial NOT NULL,
    episode_code character varying(30),
    status character varying(100),
    description text
);


ALTER TABLE public.pickup_request OWNER TO postgres;

--
-- Name: prod_setting; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE prod_setting (
    id serial NOT NULL,
    "key" character varying(100),
    value text,
    description text,
    "type" character varying(30),
    search_type character varying(200),
    category character varying(256)
);


ALTER TABLE public.prod_setting OWNER TO postgres;

--
-- Name: render; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE render (
    id serial NOT NULL,
    search_type character varying(100),
    search_id integer,
    images text,
    "session" text,
    "login" character varying(100) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT now(),
    version smallint,
    snapshot_code character varying(30),
    file_range character varying(200)
);


ALTER TABLE public.render OWNER TO postgres;

--
-- Name: script; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE script (
    id serial NOT NULL,
    files text,
    episode_code character varying(30) NOT NULL,
    author character varying(200) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT now(),
    code character varying(30),
    description text,
    stage character varying(256),
    title text
);


ALTER TABLE public.script OWNER TO postgres;

--
-- Name: series; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE series (
    id serial NOT NULL,
    code character varying(30) NOT NULL,
    name character varying(100) NOT NULL,
    images text,
    description text,
    "timestamp" timestamp without time zone DEFAULT now(),
    initials character varying(4)
);


ALTER TABLE public.series OWNER TO postgres;

--
-- Name: shot; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE shot (
    id serial NOT NULL,
    episode_code character varying(30) NOT NULL,
    code character varying(30) NOT NULL,
    description text,
    status text,
    discussion text,
    tc_frame_start smallint DEFAULT 1,
    tc_frame_end smallint DEFAULT 1,
    "timestamp" timestamp without time zone DEFAULT now(),
    snapshot text,
    pipeline_code character varying(30),
    parent_code character varying(30),
    complexity smallint,
    s_status character varying(30),
    sort_order smallint,
    priority character varying(30)
);


ALTER TABLE public.shot OWNER TO postgres;

--
-- Name: shot_audio; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE shot_audio (
    id serial NOT NULL,
    title character varying(30),
    shot_code character varying(100)
);


ALTER TABLE public.shot_audio OWNER TO postgres;

--
-- Name: storyboard; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE storyboard (
    id serial NOT NULL,
    files text,
    episode_code character varying(30) NOT NULL,
    artist character varying(200) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT now(),
    code character varying(30),
    shot_code character varying(30),
    description text
);


ALTER TABLE public.storyboard OWNER TO postgres;

--
-- Name: submission; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE submission (
    id serial NOT NULL,
    search_type character varying(200),
    search_id integer,
    snapshot_code character varying(30),
    context character varying(100),
    version integer,
    description text,
    "login" character varying(30),
    "timestamp" timestamp without time zone DEFAULT now(),
    s_status character varying(30),
    artist character varying(256),
    status character varying(100)
);


ALTER TABLE public.submission OWNER TO postgres;

--
-- Name: submission_in_bin; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE submission_in_bin (
    id serial NOT NULL,
    submission_id integer NOT NULL,
    bin_id integer NOT NULL
);


ALTER TABLE public.submission_in_bin OWNER TO postgres;

--
-- Name: test; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE test (
    id serial NOT NULL,
    s_status character varying(30),
    discussion text,
    foo integer
);


ALTER TABLE public.test OWNER TO postgres;

--
-- Name: asset_library_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY asset_library
    ADD CONSTRAINT asset_library_pkey PRIMARY KEY (code);


--
-- Name: asset_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY asset
    ADD CONSTRAINT asset_pkey PRIMARY KEY (code);


--
-- Name: asset_type_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY asset_type
    ADD CONSTRAINT asset_type_pkey PRIMARY KEY (code);


--
-- Name: bin_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY bin
    ADD CONSTRAINT bin_pkey PRIMARY KEY (id);


--
-- Name: composite_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY composite
    ADD CONSTRAINT composite_pkey PRIMARY KEY (id);


--
-- Name: episode_code_line_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY final_wave
    ADD CONSTRAINT episode_code_line_key UNIQUE (episode_code, line);


--
-- Name: episode_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY episode
    ADD CONSTRAINT episode_pkey PRIMARY KEY (code);


--
-- Name: final_wave_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY final_wave
    ADD CONSTRAINT final_wave_pkey PRIMARY KEY (id);


--
-- Name: funpack_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY funpack
    ADD CONSTRAINT funpack_pkey PRIMARY KEY (id);


--
-- Name: instance_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY instance
    ADD CONSTRAINT instance_pkey PRIMARY KEY (id);


--
-- Name: instance_shot_code_type_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY instance
    ADD CONSTRAINT instance_shot_code_type_key UNIQUE (shot_code, name, "type");


--
-- Name: layer_instance_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY layer_instance
    ADD CONSTRAINT layer_instance_pkey PRIMARY KEY (id);


--
-- Name: layer_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY layer
    ADD CONSTRAINT layer_pkey PRIMARY KEY (id);


--
-- Name: layer_shot_code_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY layer
    ADD CONSTRAINT layer_shot_code_key UNIQUE (shot_code, name);


--
-- Name: naming_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY naming
    ADD CONSTRAINT naming_pkey PRIMARY KEY (id);


--
-- Name: nat_pause_episode_code_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY nat_pause
    ADD CONSTRAINT nat_pause_episode_code_key UNIQUE (episode_code);


--
-- Name: nat_pause_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY nat_pause
    ADD CONSTRAINT nat_pause_pkey PRIMARY KEY (id);


--
-- Name: pickup_request_episode_code_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY pickup_request
    ADD CONSTRAINT pickup_request_episode_code_key UNIQUE (episode_code);


--
-- Name: pickup_request_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY pickup_request
    ADD CONSTRAINT pickup_request_pkey PRIMARY KEY (id);


--
-- Name: prod_setting_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY prod_setting
    ADD CONSTRAINT prod_setting_pkey PRIMARY KEY (id);


--
-- Name: render_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY render
    ADD CONSTRAINT render_pkey PRIMARY KEY (id);


--
-- Name: scene_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY shot
    ADD CONSTRAINT scene_pkey PRIMARY KEY (code);


--
-- Name: script_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY script
    ADD CONSTRAINT script_pkey PRIMARY KEY (id);


--
-- Name: series_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY series
    ADD CONSTRAINT series_pkey PRIMARY KEY (code);


--
-- Name: shot_audio_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY shot_audio
    ADD CONSTRAINT shot_audio_pkey PRIMARY KEY (id);


--
-- Name: storyboard_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY storyboard
    ADD CONSTRAINT storyboard_pkey PRIMARY KEY (id);


--
-- Name: submission_id_bin_id_key; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY submission_in_bin
    ADD CONSTRAINT submission_id_bin_id_key UNIQUE (submission_id, bin_id);


--
-- Name: submission_in_bin_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY submission_in_bin
    ADD CONSTRAINT submission_in_bin_pkey PRIMARY KEY (id);


--
-- Name: submission_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY submission
    ADD CONSTRAINT submission_pkey PRIMARY KEY (id);


--
-- Name: test_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY test
    ADD CONSTRAINT test_pkey PRIMARY KEY (id);


--
-- Name: shot_code_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX shot_code_idx ON shot USING btree (code);


--
-- Name: shot_episode_code_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX shot_episode_code_idx ON shot USING btree (episode_code);


--
-- Name: shot_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX shot_id_idx ON shot USING btree (id);


--
-- Name: shot_s_status_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX shot_s_status_idx ON shot USING btree (s_status);


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY shot
    ADD CONSTRAINT "$1" FOREIGN KEY (episode_code) REFERENCES episode(code) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY funpack
    ADD CONSTRAINT "$1" FOREIGN KEY (episode_code) REFERENCES episode(code) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY layer_instance
    ADD CONSTRAINT "$1" FOREIGN KEY (layer_id) REFERENCES layer(id) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY layer_instance
    ADD CONSTRAINT "$2" FOREIGN KEY (asset_code) REFERENCES asset(code) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: asset_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY instance
    ADD CONSTRAINT asset_code_fkey FOREIGN KEY (asset_code) REFERENCES asset(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: episode_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY final_wave
    ADD CONSTRAINT episode_code_fkey FOREIGN KEY (episode_code) REFERENCES episode(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: episode_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY pickup_request
    ADD CONSTRAINT episode_code_fkey FOREIGN KEY (episode_code) REFERENCES episode(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: shot_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY storyboard
    ADD CONSTRAINT shot_code_fkey FOREIGN KEY (shot_code) REFERENCES shot(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: shot_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY instance
    ADD CONSTRAINT shot_code_fkey FOREIGN KEY (shot_code) REFERENCES shot(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: shot_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY layer
    ADD CONSTRAINT shot_code_fkey FOREIGN KEY (shot_code) REFERENCES shot(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: shot_code_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY shot_audio
    ADD CONSTRAINT shot_code_fkey FOREIGN KEY (shot_code) REFERENCES shot(code) ON UPDATE CASCADE ON DELETE RESTRICT DEFERRABLE INITIALLY DEFERRED;


--
-- Name: submission_in_bin_bin_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY submission_in_bin
    ADD CONSTRAINT submission_in_bin_bin_id_fkey FOREIGN KEY (bin_id) REFERENCES bin(id) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: submission_in_bin_submission_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY submission_in_bin
    ADD CONSTRAINT submission_in_bin_submission_id_fkey FOREIGN KEY (submission_id) REFERENCES submission(id) ON UPDATE CASCADE ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- Name: pga_forms; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_forms FROM PUBLIC;
REVOKE ALL ON TABLE pga_forms FROM postgres;
GRANT ALL ON TABLE pga_forms TO postgres;
GRANT ALL ON TABLE pga_forms TO PUBLIC;


--
-- Name: pga_layout; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_layout FROM PUBLIC;
REVOKE ALL ON TABLE pga_layout FROM postgres;
GRANT ALL ON TABLE pga_layout TO postgres;
GRANT ALL ON TABLE pga_layout TO PUBLIC;


--
-- Name: pga_queries; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_queries FROM PUBLIC;
REVOKE ALL ON TABLE pga_queries FROM postgres;
GRANT ALL ON TABLE pga_queries TO postgres;
GRANT ALL ON TABLE pga_queries TO PUBLIC;


--
-- Name: pga_reports; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_reports FROM PUBLIC;
REVOKE ALL ON TABLE pga_reports FROM postgres;
GRANT ALL ON TABLE pga_reports TO postgres;
GRANT ALL ON TABLE pga_reports TO PUBLIC;


--
-- Name: pga_schema; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_schema FROM PUBLIC;
REVOKE ALL ON TABLE pga_schema FROM postgres;
GRANT ALL ON TABLE pga_schema TO postgres;
GRANT ALL ON TABLE pga_schema TO PUBLIC;


--
-- Name: pga_scripts; Type: ACL; Schema: public; Owner: postgres
--

REVOKE ALL ON TABLE pga_scripts FROM PUBLIC;
REVOKE ALL ON TABLE pga_scripts FROM postgres;
GRANT ALL ON TABLE pga_scripts TO postgres;
GRANT ALL ON TABLE pga_scripts TO PUBLIC;


--
-- PostgreSQL database dump complete
--

