###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['ProjectTemplateInstallerCmd', 'ProjectTemplateCreatorCmd', 'ProjectTemplateCheckCmd']

import tacticenv

from pyasm.common import Xml, Environment, TacticException, Common
from pyasm.search import Search, SearchType
from pyasm.biz import Project

from pyasm.command import Command

import os, shutil
import re


class ProjectDeleteCmd(Command):
    '''This will delete a project completely'''

    def execute(my):


        # drop database pg

        # DATA
        # delete from file where project_code = 'pg'
        # delete from snapshot where project_code = 'pg'
        # delete from task where project_code = 'pg'
        # delete from work_hour where project_code = 'pg'
        # delete from note where project_code = 'pg'
        # delete from wdg_settings where project_code = 'pg'

        # configuration
        # delete from schema where code = 'pg'
        # delete from pipeline where project_code = 'pg'
        # delete from search_object where namespace = 'pg'
        pass


class ProjectTemplateCreatorCmd(Command):

    def execute(my):

        my.project_code = my.kwargs.get("project_code")
        assert my.project_code

        # project code can be called anything, and we want to have a _template suffix for the template code
        my.plugin_code = "%s_template" % my.project_code

        my.template_project_code = re.sub( '_template$', '', my.plugin_code)

        my.project = Project.get_by_code(my.project_code)
        if not my.project:
            raise TacticException('This project [%s] does not exist'%my.project_code)

        my.project_type = my.project.get_value("type")
        if not my.project_type:
            my.project_type = my.project_code
        Project.set_project(my.project_code)

        my.export_template()


    def export_template(my):

        xml = Xml()
        my.xml = xml

        xml.create_doc("manifest")
        manifest_node = xml.get_root_node()
        xml.set_attribute(manifest_node, "code", my.plugin_code)

        # dump the project entry
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/project['code','%s'])" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/project")
        xml.set_attribute(data_node, "unique", "true")

        # dump the project_type entry
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/project['code','%s'].sthpw/project_type)" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/project_type")
        xml.set_attribute(data_node, "unique", "true")


        # dump the schema entry
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/schema['code','%s'])" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/schema")
        xml.set_attribute(data_node, "unique", "true")

        # find the project template search types
        namespace = my.project_type
        config_search_types = Search.eval("@GET(sthpw/search_object['namespace','%s'].search_type)" % namespace)

        #project_types = Search.eval("@GET(sthpw/search_object['namespace','%s'].search_type)" % my.project_code)

        # just dump the definition for data
        for search_type in config_search_types:
            data_node = xml.create_element("search_type")
            xml.append_child(manifest_node, data_node)
            xml.set_attribute(data_node, "code", search_type)


        search_types = [
            "config/custom_script",
            "config/widget_config",
            "config/naming",
            "config/client_trigger",
            "config/process",
            "config/trigger",
            "config/url",

            #"config/ingest_rule",
            #"config/ingest_session",
        ]

        for search_type in search_types:
            data_node = xml.create_element("sobject")
            xml.append_child(manifest_node, data_node)
            xml.set_attribute(data_node, "search_type", search_type)

            # find the currval 
            st_obj = SearchType.get(search_type)
            # have to call nextval() to initiate this sequence in the session in psql since Postgres 8.1
            seq_id = st_obj.sequence_nextval()
            
            seq_id = st_obj.sequence_currval()

            seq_id -= 1
            if seq_id > 0:
                st_obj.sequence_setval(seq_id)
            xml.set_attribute(data_node, "seq_max", seq_id)
            
            #xml.set_attribute(data_node, "path", "data.spt")


       



        # dump the login_groups entries
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/login_group['project_code','%s'])" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/login_group")
        xml.set_attribute(data_node, "unique", "true")

        # dump the pipelines entries
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/pipeline['project_code','%s'])" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/pipeline")
        xml.set_attribute(data_node, "unique", "true")

        # dump the notification entries
        data_node = xml.create_element("sobject")
        xml.append_child(manifest_node, data_node)
        xml.set_attribute(data_node, "expression", "@SOBJECT(sthpw/notification['project_code','%s'])" % my.project_code)
        xml.set_attribute(data_node, "search_type", "sthpw/notification")




        # create a virtual plugin
        plugin = SearchType.create("config/plugin")
        plugin.set_value("version", "1.0.0")
        plugin.set_value("code", "%s_project" % my.project_code)

        from plugin import PluginCreator

        base_dir = Environment.get_template_dir()

        creator = PluginCreator( base_dir=base_dir, plugin=plugin, manifest=xml.to_string() )
        creator.execute()



class ProjectTemplateInstallerCmd(Command):
    '''Install a template project thru a zip file or the unzipped folder'''
    def execute(my):

        my.mode = my.kwargs.get("mode")
        if not my.mode:
            my.mode = 'copy'
        # if a path is specified, then handle this
        my.path = my.kwargs.get("path")
        my.project_code = my.kwargs.get("project_code")
        my.is_template = my.kwargs.get("is_template")
        # check to see if the project already exists
        # FIXME: how to determine which project code? pass it in even with path kwarg for now
        project = Project.get_by_code(my.project_code)
        if project:
            raise TacticException("Project [%s] already exists in this installation. Exiting..." % my.project_code)

        if my.path:
            my.handle_path(my.path)



        assert my.project_code

        # determines which template to use
        my.template_code = my.kwargs.get("template_code")
        if not my.template_code:
            my.template_code = my.project_code

       

        # template code can end with _template or not depending if it's coming from a zip file
        if my.template_code.endswith("_template"):
            my.plugin_code = my.template_code
        else:
            my.plugin_code = "%s_template" % my.template_code

        my.template_project_code = re.sub( '_template$', '', my.template_code)
        my.force_database = my.kwargs.get("force_database")
    
        my.import_template()




    def import_template(my):

        base_dir = Environment.get_template_dir()
        template_dir = "%s/%s" % (base_dir, my.plugin_code)

        # if the template dir does not exist, then see if the template project exists in db
        if my.force_database or not os.path.exists(template_dir):
            # check to see if this template project is in the database
           
            template = Project.get_by_code(my.template_project_code)
            
            if template:
                cmd = ProjectTemplateCreatorCmd(project_code=my.template_project_code)
                cmd.execute()
            else:
                if not os.path.exists(template_dir):
                    raise TacticException("Project template [%s] is not installed. Ensure the zip file is placed in the templates directory. And you need to run the -f <zip_path> option if using create_template.py" % my.template_code)


        # read the manifest file
        manifest_path = "%s/manifest.xml" % (template_dir)
        my.xml = Xml()
        my.xml.read_file(manifest_path)


        # check to see if the database exists
        # It is assumed that the project is created in the default database
        from pyasm.search import DbContainer, DatabaseImpl
        impl = DatabaseImpl.get()
        exists = impl.database_exists(my.project_code)
        if exists:
            msg = "WARNING: Database [%s] already exists" % my.project_code
            print msg
            raise TacticException(msg)

        # this is the overriding factor:
        if my.is_template == True:
            title = Common.get_display_title(my.project_code)
        elif my.is_template == False:
            title = Common.get_display_title(my.project_code)
        elif my.is_template == None:
            # these 2 is for old usage using the command line script create_template.py
            if my.template_project_code != my.project_code:
                my.is_template = False
                title = Common.get_display_title(my.project_code)
            else:
                my.is_template = True
                title = Common.get_display_title(my.template_project_code)

        # create the project
        from create_project_cmd import CreateProjectCmd


        project_image_path = my.kwargs.get("project_image_path")

        # the project_type will get updated properly by the PluginInstaller
        # but that break the ties to the project_type entry created though, which is ok
        creator = CreateProjectCmd(
            project_code=my.project_code,
            project_title=title,
            project_type=my.template_project_code,
            is_template=my.is_template,
            use_default_side_bar=False,
            project_image_path=project_image_path
        )
        creator.execute()
        Project.set_project(my.project_code)


        # import from a plugin
        kwargs = {
            'base_dir': base_dir,
            'manifest': my.xml.to_string()
        }
        #if my.template_project_code != my.project_code:
        # should run at all times
        kwargs['filter_line_handler'] = my.filter_line_handler
        kwargs['filter_sobject_handler'] = my.filter_sobject_handler


        from plugin import PluginCreator, PluginInstaller
        installer = PluginInstaller( **kwargs )
        installer.execute()


    def handle_path(my, src_path):

        src_path = src_path.replace("\\", "/")

        # upload folder
        basename = os.path.basename(src_path)
        plugin_base_dir = Environment.get_plugin_dir()

        if my.mode =='copy':
            target_path = src_path
            target_dir = os.path.dirname(target_path)
        else:
            target_dir = Environment.get_upload_dir()
            target_path = "%s/%s" % (target_dir, basename)
        
        #plugin_dir = Environment.get_plugin_dir()
        base_dir = Environment.get_template_dir()
        template_dir = "%s/%s" % (base_dir, my.project_code)
        

        if os.path.exists(template_dir):
            raise TacticException("Template is already installed at [%s]" %template_dir)


        # unzip the file
        from pyasm.common import ZipUtil
        # this is fixed for windows if zipping doesn't use compression
        paths = ZipUtil.extract(target_path)
        """
        # TODO: The ZipUtil module errors when extracting the zip file.
        # Error:  unpack requires a string argument of length 46
        if os.name != 'nt':
            paths = ZipUtil.extract(target_path)
        else:
            src_path_without_zip_nor_version = src_path[0:-10]
            if not os.path.exists(src_path_without_zip_nor_version):

                print
                print 'To install a project template plugin on a Windows machine,'
                print 'please first unzip the TACTIC plugin file to its current directory:'
                print
                print '       ' + src_path_without_zip_nor_version
                print
                print 'and try again.'
                exit()
            shutil.move(src_path_without_zip_nor_version, upload_dir)
        """
        # veryify that the paths extracted are the expected ones
        rootname, ext = os.path.splitext(basename)
        #for path in paths:
        #    if not path.startswith("%s/" % rootname):
        #        raise TacticException("File in zip is not contained in [%s]" % rootname)

        #plugin_path = "%s/%s" % (plugin_base_dir, basename)

        # check if it unzips at the templates folder directly
        unzip_at_template_dir = False
        # move the plugin zip file to the appropriate folder
        if my.mode == 'copy':
            # if they manually drop the zip file already here, skip
            if target_dir != base_dir:
                shutil.copy(target_path, base_dir)
            else:
                unzip_at_template_dir = True
        else:
            shutil.move(target_path, base_dir)


        # move unzipped files into the plugin area
        # remove any version info, only allow 1 particular version installed for now
        import re
        rootname = re.sub('(.*)(-)(\d.*)', r'\1', rootname)
        unzip_path = "%s/%s" % (target_dir, rootname)
       
        
        dest_dir = '%s/%s'%(base_dir, rootname)
        
        if not unzip_at_template_dir and os.path.exists(dest_dir):
            shutil.rmtree(dest_dir)

            shutil.move(unzip_path, dest_dir)
        

        



    def filter_sobject_handler(my, sobject):

        search_type = sobject.get_base_search_type()

        if search_type == 'sthpw/project':
            project = Project.get_by_code(my.project_code)
            if project:
                sobject.set_value("id", project.get_id() )

            # change the code of the project
            sobject.set_value("code", my.project_code)

            title = Common.get_display_title(my.project_code)
            sobject.set_value("title", title)

            if my.is_template:
                sobject.set_value("is_template", True)
            else:
                sobject.set_value("is_template", False)

        elif search_type == 'sthpw/schema':
            sobject.set_value("code", my.project_code)

        elif search_type == 'sthpw/notification':
            sobject.set_value("project_code", my.project_code)
            sobject.set_value("code", "")

        elif search_type in ['sthpw/pipeline']:
            sobject.set_value("project_code", my.project_code)
            if my.template_project_code != my.project_code:
                # get the old code
                old_code = sobject.get_code()
                if old_code.startswith("%s/" % my.template_project_code):
                    new_code = old_code.replace("%s/" % my.template_project_code, "%s/" % my.project_code)
                else:
                    new_code = "%s/%s" % (my.project_code, old_code)
                sobject.set_value("code", new_code)




        return sobject


    def filter_line_handler(my, path, line):
        '''NOT used now'''
        return line

        # this is only called if the project code is different from the
        # template code

        file_name = os.path.basename(path)

        if file_name in ['sthpw_project.spt']:
            # change codes to project code
            if line.startswith('''insert.set_value('code','''):
                line = '''insert.set_value('code', """%s""")\n''' % my.project_code
            elif line.startswith('''insert.set_value('title','''):
                title = Common.get_display_title(my.project_code)
                line = '''insert.set_value('title', """%s""")\n''' % title

            elif line.startswith('''insert.set_value('is_template','''):
                if my.is_template:
                    line = '''insert.set_value('is_template', """true""")\n'''
                else:
                    line = '''insert.set_value('is_template', """false""")\n'''



        elif file_name in ['sthpw_schema.spt']:
            if line.startswith('''insert.set_value('code','''):
                line = '''insert.set_value('code', """%s""")\n''' % my.project_code

        elif file_name in ['sthpw_pipeline.spt']:
            if line.startswith('''insert.set_value('project_code','''):
                line = '''insert.set_value('project_code', """%s""")\n''' % my.project_code

        return line







class ProjectTemplateCheckCmd(Command):
    '''This will check the integrity of a project to see if is suitable
    for export as a distributable project template'''

    def execute(my):

        my.project_code = my.kwargs.get("project_code")
        my.prefix = my.kwargs.get("prefix")


        my.project = Project.get_by_code(my.project_code)
        my.project_type = my.project.get_value("type")

        my.check_project()
        my.check_search_type()



    def check_project(my):


        # check that the project code starts with the prefix
        if not my.project.get_code().startswith("%s_" % my.prefix):
            raise TacticException("Project code [%s] does not start with prefix [%s]" % (my.project_code, my.prefix) )


        # check that the project type is the same as the project code
        if not my.project_code != my.project_type:
            raise TacticException("Project code [%s] does not match the project_type [%s]" % (my.project_code, my.project_type) )



    def check_search_type(my):

        # all search objects in the namespace of <project_code> should
        # start with the prefix


        search = Seach("sthpw/search_type")
        search.add_filter("namespace", my.project_type)
        search_types = search.get_sobjects()


        for search_type in search_types:
            if search_type.get_value("search_type").startswith("%s_" % my.prefix):
                raise TacticException( "sType [%s] does not start with prefix [%s]" % (search_type.get_value("search_type"), my.prefix) )





if __name__ == '__main__':

    from pyasm.security import Batch
    Batch(project_code='admin')

    #cmd = ProjectTemplateCreatorCmd(project_code='pg')
    #Command.execute_cmd(cmd)

    cmd = ProjectTemplateInstallerCmd(project_code='scrum')
    Command.execute_cmd(cmd)

    #cmd = ProjectTemplateCheckCmd(project_code='di', prefix='di')
    #Command.execute_cmd(cmd)




