###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['SearchLimitWdg']

from pyasm.common import Environment
from pyasm.web import WebContainer, Widget, HtmlElement, DivWdg, SpanWdg
from pyasm.widget import HiddenWdg, SubmitWdg, SelectWdg, TextWdg, IconButtonWdg, IconSubmitWdg, IconWdg, SwapDisplayWdg
from tactic.ui.filter import FilterData
       
class SearchLimitWdg(Widget):
    DETAIL = "detail_style"
    LESS_DETAIL = "less_detail_style"
    SIMPLE = "simple_style"

    def __init__(my, name='search_limit', label="Showing", limit=20, refresh=True):
        my.search_limit_name = name
        my.label = label
        my.search_limit = limit
        my.fixed_offset = False
        my.style = my.DETAIL
        my.prefix = "search_limit"
        my.refresh = refresh
        my.refresh_script = 'spt.dg_table.search_cbk(evt, bvr)'
        if my.refresh:
            my.prev_hidden_name = 'Prev'
            my.next_hidden_name='Next'
        else:
            my.prev_hidden_name = '%s_Prev' %my.label
            my.next_hidden_name = '%s_Next' %my.label


        my.chunk_size = 0
        my.chunk_num = 0

        super(SearchLimitWdg, my).__init__()
        
    def init(my):
        my.current_offset = 0
        my.count = None
        my.text = TextWdg(my.search_limit_name)
        my.text.add_style("width: 23px")
        my.text.add_style("margin-bottom: -1px")
        my.text.add_class("spt_search_limit_text")
        my.text.set_persist_on_submit(prefix=my.prefix)

        behavior = {
                'type': 'keydown',
                'cbjs_action': '''
                 if (evt.key=='enter') {
                    // register this as changed item
                    var value = bvr.src_el.value;
                    
                        if (isNaN(value) || value.test(/[\.-]/)) {
                            spt.error('You have to use an integer.');
                        }
                }
        '''}

        my.text.add_behavior(behavior)


        # get the search limit that is passed in
        filter_data = FilterData.get()
        values = filter_data.get_values_by_prefix(my.prefix)
        if not values:
            # check web for embedded table
            web = WebContainer.get_web()
            values = {}
            limit_value = web.get_form_value("search_limit")
            label_value = web.get_form_value(my.label)
            if limit_value:
                values['search_limit'] = limit_value
            if label_value:
                values[my.label] = label_value
        else:
            values = values[0]
        my.stated_search_limit = values.get("search_limit")
        if my.stated_search_limit:
            my.stated_search_limit = int(my.stated_search_limit)
        else:
            my.stated_search_limit = 0
      
        # reused for alter_search() later
        my.values = values
       
    def set_refresh_script(my, script):
        my.refresh_script = script

    def set_label(my, label):
        my.label = label

    def get_limit(my):
        return my.search_limit

    def get_stated_limit(my):
        return my.stated_search_limit


    def set_limit(my, limit):
        if limit <= 20:
            limit = 20
        my.search_limit = limit

    def set_offset(my, offset):
        my.fixed_offset = True
        my.current_offset = offset


    def set_style(my, style):
        my.style = style


    def set_chunk(my, chunk_size, chunk_num, limit=None):
        my.chunk_size = chunk_size
        my.chunk_num = chunk_num
        if limit:
            my.search_limit = limit
        else:
            my.search_limit = chunk_size

        # to avoid 0 limit on undefined search limit in fast table
        if my.search_limit == 0:
            my.search_limit = 100
        

    def alter_search(my, search):
        if not search:
            return
        my.set_search(search)
        security = Environment.get_security()
        # allow security to alter the search
        security.alter_search(search)
        search.set_security_filter()

        my.count = search.get_count()

        web = WebContainer.get_web()

        '''
        filter_data = FilterData.get()
        values = filter_data.get_values_by_prefix(my.prefix)
        if not values:
            values = {}
        else:
            values = values[0]
        '''
        values = my.values
        #search_limit = values.get("search_limit")
        # look at the stated search limit only if there is no chunk_size
        if not my.chunk_size:
            search_limit = my.stated_search_limit
            if search_limit:
                try:
                    my.search_limit = int(search_limit)
                except ValueError:
                    my.search_limit = 20
                if my.search_limit <= 0:
                    my.search_limit = 1
        
        # find out what the last offset was
        #last_search_offset = web.get_form_value("last_search_offset")
        last_search_offset = values.get("%s_last_search_offset"%my.label)
        if last_search_offset:
            last_search_offset = int(last_search_offset)
        else:
            last_search_offset = 0
        # based on the action, set the new offset
        #if web.get_form_value("Next"):
        # FIXME: Next Prev not working for embedded tables yet

        if values.get("Next"):
            current_offset = last_search_offset + my.search_limit
            if current_offset >= my.count:
                current_offset = 0

        #elif web.get_form_value("Prev"):
        elif values.get("Prev"):
            current_offset = last_search_offset - my.search_limit
            if current_offset < 0:
                current_offset = int(my.count/my.search_limit) * my.search_limit
      
        # this happens when the user jumps from Page 3 of a tab to a tab that
        # doesn't really need this widget
        elif last_search_offset > my.count:
            current_offset = 0
        #elif web.get_form_value(my.label) != "":
        elif values.get(my.label):
            #value = web.get_form_value(my.label)
            value = values.get(my.label)
            if not value.startswith("+"):
                current_offset, tmp = value.split(" - ")
                current_offset = int(current_offset) - 1
            else:
                current_offset = 0
        else:
            current_offset = 0 

        if my.fixed_offset == False:
            my.current_offset = current_offset


        # add ability to break search limit search into chunks
        #search_limit = my.search_limit
        current_offset = my.current_offset

        if my.chunk_num:
            #current_offset = my.current_offset + (my.chunk_num*my.chunk_size)
            
            current_offset = last_search_offset + (my.chunk_num*my.chunk_size)
       
        my.search.set_limit(my.search_limit)
        my.search.set_offset(current_offset)



    def get_display(my):

        web = WebContainer.get_web()

        widget = DivWdg()
        widget.add_border(modifier=10)
        widget.add_class("spt_search_limit_top")

        widget.add_style("font-size: 10px")

        widget.add_style("padding: 5px")
        #widget.add_gradient("background", "background", 10, -20, reverse=True)
        widget.set_round_corners(5)
        widget.add_style("margin-top: -4px")

        div = SpanWdg(css='med')
        div.set_attr("nowrap", "1")

        hidden = HiddenWdg("prefix", my.prefix)
        div.add(hidden)

        limit_wdg = my.text
        #limit_wdg.add_event('onchange', 'if (this.value < 20 || !Common.validate_int(this.value)) this.value=20') 
        limit_wdg.add_style("margin-top: -3px")
        limit_wdg.set_attr("size", "1")
        limit_wdg.add_attr("title", "Set number of items per page")
        limit_wdg.add_color("background", "background", -8)
        limit_wdg.add_style("font-size: 10px")

        from tactic.ui.container import DialogWdg
        limit_dialog = DialogWdg()
        limit_wdg.add(limit_dialog)
        limit_dialog.set_as_activator(limit_wdg)
        limit_dialog.add_title("Viewing")

        limit_content = DivWdg()
        limit_dialog.add(limit_content)
        limit_content.add_color("color", "color")
        limit_content.add_color("background", "background")
        limit_content.add_style("padding", "5px")
        limit_content.add_border()

        limit_content.add("Show ")

        limit_select = SelectWdg("limit_select")
        limit_select.add_class("spt_search_limit_select")
        limit_select.set_option("values", "10|20|50|100|200|Custom")
        limit_content.add(limit_select)
        limit_content.add("items per page<br/>")

        limit_select.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_search_limit_top");
            var value = bvr.src_el.value;
            var custom = top.getElement(".spt_search_limit_custom");
            if (value == 'Custom') {
                custom.setStyle("display", "");
            }
            else {
                custom.setStyle("display", "none");
            }

            '''
        } )


        custom_limit = DivWdg()
        limit_content.add(custom_limit)
        custom_limit.add_class("spt_search_limit_custom")
        custom_limit.add_style("display: none")
        custom_limit.add("<br/>Custom: ")
        text = TextWdg("custom_limit")
        text.add_class("spt_search_limit_custom_text")
        text.add_style("width: 50px")
        custom_limit.add(text)
        text.add(" items")
        behavior = {
                'type': 'keydown',
                'cbjs_action': '''
                 if (evt.key=='enter') {
                    // register this as changed item
                    var value = bvr.src_el.value;
                    if (isNaN(value) || value.test(/[\.-]/)) {
                        spt.error('You have to use an integer.');
                    }
                }
        '''}

        
        text.add_behavior(behavior)



        
        limit_content.add("<hr/>")


        #from pyasm.widget import ProdIconButtonWdg
        #button = ProdIconButtonWdg("Save")
        from tactic.ui.widget.button_new_wdg import ActionButtonWdg
        button = ActionButtonWdg(title='Search')
        limit_content.add(button)
        button.add_style("float: right")
        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_search_limit_top");
            var select = top.getElement(".spt_search_limit_select");
            var value = select.value;
            if (value == 'Custom') {
                custom = top.getElement(".spt_search_limit_custom_text");
                value = custom.value;
            }
            if (value == '') {
                value = 20;
            }
            var text = top.getElement(".spt_search_limit_text");
            text.value = value;

            spt.dg_table.search_cbk({}, bvr) 
            '''
        } )

        limit_content.add("<br clear='all'/>")

        offset_wdg = HiddenWdg("%s_last_search_offset" %my.label)
        offset_wdg.set_value(my.current_offset)
        div.add(offset_wdg)
    
        if not my.search and not my.sobjects:
            return div

        # my.count should have been set in alter_search()
        # which can be called explicitly thru this instance, my.
        if not my.count:
            my.count = my.search.get_count(no_exception=True)
        
        # if my.sobjects exist thru inheriting from parent widgets
        # or explicitly set, (this is not mandatory though)
        if my.sobjects and len(my.sobjects) < my.search_limit:
            limit = len(my.sobjects)
        elif my.search and my.count < my.search_limit:
            # this is only true if the total result of the search is 
            # less than the limit and so this wdg will not display
            limit = my.count
        else:
            limit = my.search_limit

    
        #if my.current_offset == 0 and limit < my.search_limit:
        #    return None
        if my.refresh: 
            prev = SpanWdg( IconSubmitWdg("Prev", IconWdg.LEFT, False ) )
            prev.add_style("margin-right: 2px")
            prev.add_style("margin-top: -2px")
            next = IconSubmitWdg("Next", IconWdg.RIGHT, False, icon_pos="right" )
            next.add_style("margin-left: -3px")

            prev.add_behavior( {
                'type': 'click_up',
                'cbjs_action': my.refresh_script
            } )
            next.add_behavior( {
                'type': 'click_up',
                'cbjs_action': my.refresh_script
            } )
        else: # the old code pre 2.5
            prev = IconButtonWdg("Prev", IconWdg.LEFT, False )
            hidden_name = my.prev_hidden_name
            hidden = HiddenWdg(hidden_name,"")
            prev.add(hidden)
            prev.add_event('onclick'," spt.api.Utility.get_input(document,'%s').value ='Prev';%s"\
                    %(hidden_name, my.refresh_script))
            next = IconButtonWdg("Next", IconWdg.RIGHT, False, icon_pos="right" )
            hidden_name = my.next_hidden_name
            hidden = HiddenWdg(hidden_name,"")
            next.add(hidden)
            next.add_event('onclick',"spt.api.Utility.get_input(document,'%s').value ='Next';%s" \
                    %(hidden_name, my.refresh_script))


        label_span = SpanWdg("Showing: ")
        #label_span.add_style('color','#c2895d')
        label_span.add_color('color','color2', +15)
        div.add(label_span)
        div.add( prev )
       

        # this min calculation is used so that if my.sobjects is not set
        # above for the calculation of the limit, which will make the last 
        # set of range numbers too big
        
        left_bound = my.current_offset+1
        right_bound = min(my.current_offset+limit, my.count)
        if left_bound > right_bound:
            left_bound = 1
        current_value = "%d - %d" % (left_bound, right_bound)

        if my.style == my.SIMPLE:
            div.add( current_value )
        else:
            # add a range selector using ItemsNavigatorWdg
            from pyasm.widget import ItemsNavigatorWdg
            selector = ItemsNavigatorWdg(my.label, my.count, my.search_limit)
            selector.select.add_behavior( {
                'type': 'change',
                'cbjs_action': my.refresh_script
            } )

            selector.set_style(my.style)

            selector.set_value(current_value)
            selector.set_display_label(False)

            div.add( selector) 
            #div.add( limit_wdg)

        div.add( next )


        if not my.style == my.SIMPLE:
            div.add( " x ")
            div.add( limit_wdg)

        widget.add(div)


        return widget


