###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['Test2Wdg']

from pyasm.common import Environment, Common, jsonloads
from pyasm.biz import Project
from pyasm.web import Widget, DivWdg, HtmlElement, WebContainer, Canvas

from tactic.ui.common import BaseRefreshWdg



class Test2Wdg(BaseRefreshWdg):

    ARGS_KEYS = {
    }

    def get_display(my):

        top = my.top
        top.add_gradient("background", "background", -10)

        canvas = Canvas()
        top.add(canvas)
        canvas.set_id("chart1")
        canvas.add_attr("width", "800")
        canvas.add_attr("height", "500")
        canvas.add("Your web-browser does not support the HTML 5 canvas element.")


        canvas.add_behavior( {
            'type': 'load',
            'cbjs_action': ONLOAD_JS
        } )

        canvas.add_behavior( {
            'type': 'load',
            'cbjs_action': '''
            spt.chart.top = bvr.src_el;
            '''
        } )


        labels = ['chr001', 'chr002', 'chr003', 'chr004', 'prop001', 'prop002']

        grid = Grid(labels=labels)
        top.add(grid)

        data =   [1.2, 5.5, 7.5, 5.3, 8.2, 1.1, 3.3]
        data = ChartData(data=data, color="#F00")
        top.add(data)

        data =   [9.2, 3.2, 8.8, 1.5, 3.3]
        data = ChartData(data=data, color="#0F0")
        top.add(data)

        data =   [1.1, 3.5, 2.2, 6.6, 1.3, 9.4]
        data = ChartData(data=data, color="#00F")
        top.add(data)

        data =   [18, 17, 15.5, 19, 17, 16.8, 11.4]
        data = ChartData(data=data, color="#0FF")
        top.add(data)



        for i, label in enumerate(labels):

            label_div = DivWdg()
            top.add(label_div)
            label_div.add_style("position: absolute")
            label_div.add(label)
            label_div.add_style("left: %s" % (i*50+50))
            #label_div.add_style("top: %s" % (i*50+50))
            label_div.add_style("top: %s" % 50)

            #label_div.set_round_corners()
            label_div.add_style("padding: 3px")
            label_div.add_border()
            label_div.add_color("background", "background3")
            label_div.set_box_shadow()



        return top


class Grid(BaseRefreshWdg):

    def get_display(my):


        labels = my.kwargs.get("labels")

        top = my.top
        top.add_behavior( {
            'type': 'load',
            'labels': labels,
            'cbjs_action': '''
            var size = spt.chart.get_size();
            var color = '#333';

            var origin = spt.chart.get_origin();
            var outer = spt.chart.get_outer();

            var interval = spt.chart.get_interval();

            spt.chart.draw_line( origin, {x: origin.x, y: outer.y}, color );
            spt.chart.draw_line( origin, {x: outer.x, y: origin.y}, color );


            // draw the horizontal grid lines
            for (var i = 0; ; i++) {

                var x = origin.x + i*interval.x;
                var y = origin.y;
                if (x > outer.x) break;

                var start = {x: x, y: y};
                var end = {x: x, y: y+5};
                spt.chart.draw_line( start, end, color );

            }


            for (var i = 0; i < bvr.labels.length; i++) {
                var ctx = spt.chart.get_ctx();
                ctx.fillStyle = '#000';
                ctx.font = '12px san-serif';
                ctx.textBaseline = 'bottom';
                var label = bvr.labels[i];
                ctx.fillText(label, origin.x + (i+1)*interval.x, origin.y + 20);
            }


            // draw the vertical grid lines
            for (var i = 0; ; i++) {

                var y = origin.y - i*interval.y;
                if (y < outer.y) break;

                var start = {x: origin.x-1, y: y};
                var end = {x: origin.x-5, y: y};
                spt.chart.draw_line( start, end, "#999" );

                if (i == 0) continue;

                var start = {x: origin.x+1, y: y};
                var end = {x: outer.x, y: y};
                spt.chart.draw_line( start, end, "#EEE" );

                var ctx = spt.chart.get_ctx();
                ctx.fillStyle = '#000';
                ctx.font = '12px san-serif';
                ctx.textBaseline = 'bottom';
                ctx.fillText(i, origin.x - 20, origin.y - i*interval.y+8);

            }

            '''
        } )

        return top




class ChartData(BaseRefreshWdg):

    def get_display(my):

        top = my.top

        data = my.kwargs.get("data")
        labels = my.kwargs.get("labels")
        color = my.kwargs.get("color")


        top.add_behavior( {
            'type': 'load',
            'data': data,
            'color': color,
            'cbjs_action': '''
            var size = spt.chart.get_size();

            var origin = spt.chart.get_origin();
            var outer = spt.chart.get_outer();

            var interval = spt.chart.get_interval();

            var last = origin;
            for (var i = 0; i < bvr.data.length; i++) {

                var x = (i+1)*interval.x;
                var y = bvr.data[i]*interval.y;

                var cur = spt.chart.get_pos(x, y);
                spt.chart.draw_line( last, cur, bvr.color );

                spt.chart.draw_dot( x, y, 3, bvr.color );


                last = cur;
            }

            '''
        } )

        return top



ONLOAD_JS = '''

spt.chart = {}

spt.chart.top = null;

spt.chart.get_top = function() {
    return spt.chart.top;
}



spt.chart.get_size = function() {
    var top = spt.chart.get_top();
    return top.getSize();
}



spt.chart.get_origin = function() {
    var size = spt.chart.get_size();
    var offset = 50;
    var origin = {
        x: offset,
        y: size.y - offset
    }
    return origin;
}


spt.chart.get_outer = function() {
    var size = spt.chart.get_size();
    var offset = 50;
    var outer = {
        x: size.x - offset,
        y: offset 
    }
    return outer;
}


spt.chart.get_pos = function(x, y) {
    var origin = spt.chart.get_origin();
    var pos = {
        x: origin.x + x,
        y: origin.y - y
    }
    return pos;
}


spt.chart.get_interval = function() {
    var origin = spt.chart.get_origin();
    var outer = spt.chart.get_outer();
    var max = {
      x: 8,
      y: 25
    }

    var interval = {};

    var grid_height = origin.y - outer.y;
    interval.y = parseInt(grid_height / max.y);

    var grid_width = outer.x - origin.x;
    interval.x = parseInt(grid_width / max.x);

    return interval;
}



spt.chart.get_ctx = function() {
    var top = spt.chart.get_top();
    //var chart = top.getElement(".spt_chart");
    var ctx = top.getContext('2d');
    return ctx;
}
 
spt.chart.draw_line = function(start, end, color) {
    var ctx = spt.chart.get_ctx();
    ctx.strokeStyle = color;  
    ctx.lineWidth = 2;
    ctx.beginPath();
    ctx.moveTo(start.x, start.y);
    ctx.lineTo(end.x, end.y);

    ctx.stroke();
}


spt.chart.draw_dot = function(x, y, size, color) {

    var pos = spt.chart.get_pos(x, y);

    var ctx = spt.chart.get_ctx();
    ctx.fillStyle = color;
    ctx.beginPath();
    ctx.arc(pos.x, pos.y, size, 0, Math.PI*2, true);
    ctx.closePath();
    ctx.fill();
}


        '''

