###########################################################
#
# Copyright (c) 2011, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['SnapshotFilesWdg']

from pyasm.common import Environment
from pyasm.search import SearchKey
from pyasm.biz import Snapshot
from pyasm.widget import IconWdg

from tactic.ui.common import BaseRefreshWdg
from tactic.ui.tools import RepoBrowserDirListWdg
from tactic.ui.widget import DirListWdg
from tactic.ui.container import DialogWdg, Menu, MenuItem, SmartMenu

import os


class SnapshotFilesWdg(BaseRefreshWdg):
    '''This is used in the "Files" hidden row to display all the files of
    a snapshot'''

    def get_files(my):

        paths = []

        # remember this here for now
        my.files = []


        search_key = my.kwargs.get("search_key")
        sobject = SearchKey.get_by_search_key(search_key)
        # if it is deleted, return
        if not sobject:
            return []
    
        if isinstance(sobject, Snapshot):
            snapshots = [sobject]
        else:
            snapshots = Snapshot.get_by_sobject(sobject, "publish")

        for snapshot in snapshots:
            snapshot_paths = snapshot.get_all_lib_paths()

            files = snapshot.get_all_file_objects()
            my.files.extend(files)

            for path in snapshot_paths:

                # if the path is a directory, get all of the files
                if os.path.isdir(path):
                    path = "%s/" % path
                    paths.append(path)

                    dirlist = os.listdir(path)
                    for item in dirlist:
                        item_path = "%s%s" % (path, item)
                        paths.append(item_path)

                else:
                    paths.append(path)

        return paths




    def get_display(my):

        top = my.top
        top.add_style("padding: 10px")
        top.add_color("background", "background", -5)

        paths = my.get_files()
        base_dir = Environment.get_asset_dir()

        dir_list = SnapshotDirListWdg(base_dir=base_dir, location="server", show_base_dir=True,paths=paths, all_open=True, files=my.files)

        top.add(dir_list)


        return top






class SnapshotDirListWdg(DirListWdg):

    def add_top_behaviors(my, top):

        # convert this to a repo directory
        asset_dir = Environment.get_asset_dir()

        # FIXME: not sure how general this
        #webdirname = "/assets/%s" % dirname.replace(asset_dir, "")

        top.add_behavior( {
        'type': 'smart_click_up',
        'bvr_match_class': 'spt_dir_list_item',
        'cbjs_action': '''
        var path = bvr.src_el.getAttribute("spt_path");
        var asset_dir = '%s';
        var url = "/assets/" + path.replace(asset_dir, "");
        window.open(url);
        ''' % asset_dir
        } )


        # add a top menu
        menu = Menu(width=180)
        menu_item = MenuItem(type='title', label='Actions')
        menu.add(menu_item)
        menu_item = MenuItem(type='action', label='Check-out To Sandbox')
        menu.add(menu_item)
        menu_item.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''spt.alert('Not implemented yet.')'''
        } )
        menu_item = MenuItem(type='action', label='Copy to Clipboard')
        menu.add(menu_item)
        menu_item.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        var activator = spt.smenu.get_activator(bvr);
        var path = activator.getAttribute("spt_path");

        var search_key = activator.getAttribute("spt_file_search_key");

        var server = TacticServerStub.get();
        var class_name = 'tactic.command.clipboard_cmd.ClipboardCopyCmd';
        var search_keys = [search_key];
        var kwargs = {
            search_keys: search_keys
        }
        try {
            spt.app_busy.show("Copy to Clipboard ...");
            server.execute_cmd(class_name, kwargs);
            spt.app_busy.hide();
            }
        catch(e) {
            spt.alert(spt.exception.handler(e));
        }
        '''
        } )

        menus_in = {
            'FILE_MENU_CTX': menu,
        }
        SmartMenu.attach_smart_context_menu( top, menus_in, False )
 
        super(SnapshotDirListWdg, my).add_top_behaviors(top)




    def add_file_behaviors(my, item_div, dirname, basename):

        file_objects = my.kwargs.get("files")

        if my.file_count >= len(file_objects):
            file_object = file_objects[0]
        else:
            file_object = file_objects[my.file_count]
        file_search_key = file_object.get_search_key()

        path = "%s/%s" % (dirname, basename)
        item_div.add_class("spt_dir_list_item")
        item_div.add_attr("spt_path", path)
        item_div.add_attr("spt_file_search_key", file_search_key)
        SmartMenu.assign_as_local_activator( item_div, 'FILE_MENU_CTX' )

        return


        # FIXME: this isn't very efficient
        files = my.kwargs.get("files")
        for file in files:
            if file.get_value("file_name") == basename:
                break
        else:
            file = None

        location = file.get_value("location")
        md5 = file.get_value("md5")
        if location:
            item_div.add(location)
        if md5:
            item_div.add(md5)




    def get_file_icon(my, dir, item):
        return IconWdg.DETAILS

    def get_dir_icon(my, dir, item):
        return IconWdg.LOAD




