###########################################################
#
# Copyright (c) 2009, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['ProcessGroupSelectWdg', 'LoginTableElementWdg']

from pyasm.search import Search, SearchKey
from pyasm.biz import Pipeline
from pyasm.web import DivWdg
from pyasm.widget import BaseInputWdg, SelectWdg, SimpleTableElementWdg

class ProcessGroupSelectWdg(BaseInputWdg):
    '''This widget builds a select widget '''
    
    def get_display(my):
        from tactic.ui.panel import EditWdg
        if hasattr(my, 'parent_wdg') and isinstance(my.get_parent_wdg(), EditWdg):
            sobject = my.get_current_sobject()
            parent = sobject.get_parent()
            group = None
            pipeline_code = None

            if parent:
                pipeline_code = parent.get_value('pipeline_code')
            pipeline = Pipeline.get_by_code(pipeline_code)
            if pipeline:
                attrs = pipeline.get_process_attrs(sobject.get_value('process'))
                group = attrs.get('%s_login_group'%my.get_name())
            if group:
            
                group_users = Search.eval("@GET(sthpw/login_group['login_group', '%s'].sthpw/login_in_group.sthpw/login.login)"%group)
            else:
                group_users = Search.eval("@GET(sthpw/login.login)")
            select = SelectWdg(my.get_input_name())
            select.add_empty_option("-- Select a User --")
            values = []
            labels = []
            for user in group_users:
                values.append(user)
                labels.append('  %s'%user)
            select.set_option('values', values)
            select.set_option('labels', labels)
            current_value = sobject.get_value(my.get_name())
            if current_value:
                select.set_value(current_value)
            return select


        all_users = Search.eval("@GET(sthpw/login.login)")
        '''
        groups = Search.eval("@SOBJECT(sthpw/login_group)")^M
        group_dict = {}^M
        for group in groups:^M
            group_users =
Search.eval("@GET(sthpw/login_group['login_group',
'%s'].sthpw/login_in_group.sthpw/login.login)"%group.get_value('login_group'))^M
            group_dict[group.get_value('login_group')] = group_users^M
        '''

        logins_dict = {}
        for user in all_users:
            logins_dict[user] = {}
        group_dict = {}
        items = Search.eval("@SOBJECT(sthpw/login_in_group)")
        for item in items:
             item_login = item.get_value("login")
             if logins_dict.get(item_login) == None:
                 continue
             item_group = item.get_value("login_group")

             group_list = group_dict.get(item_group)
             if group_list == None:
                 group_list = []
                 group_dict[item_group] = group_list
             group_list.append(item_login)


            
        
        top = DivWdg()
        
        top.add_class("spt_input_top")

        # HACK! This isn't very well constructed
        ### Tore: Not my code! Copied from ProcessContextInputWdg. Seems to work though.
        top.add_attr("spt_cbjs_get_input_key", "return cell_to_edit.getAttribute('spt_pipeline_code');")
        
        # Adding an "all users" select option in case it can't find a useful select widget.
        div = DivWdg()
        div.add_class("spt_input_option")
        #div.add_attr("spt_input_key", '__all__') #Not needed, since it defaults to the first one anyway.
        select = SelectWdg(my.get_name())
        select.add_empty_option("-- Select a User --")
        values = []
        labels = []
        for user in all_users:
            values.append(user)
            labels.append('%s'%user)

        # -- NOTE: leaving this commented out code here for reference. Not sure why this is the case but when
        # --       this click behavior is used instead of a 'change' behavior that forces a blur on select,
        # --       click selection only works for this widget in Firefox and does NOT work in IE
        #
        # select.add_behavior( { 'type': 'click',
        #    'cbjs_action': 'spt.dg_table.select_wdg_clicked( evt, bvr.src_el );' } )


        # -- Replace previous 'click' behavior with a 'change' behavior to force blur() of select element ...
        # -- this works for both Firefox and IE
        #
        select.add_behavior( { 'type': 'change',
           'cbjs_action': 'bvr.src_el.blur();' } )


        behavior = {
            'type': 'keyboard',
            'kbd_handler_name': 'DgTableSelectWidgetKeyInput',
        }
        select.add_behavior( behavior )


        select.set_option("values", values)
        select.set_option("labels", labels)
           
        div.add(select)
        top.add(div)
        
        #Building each of the select widgets per group here.
        for group in group_dict.keys():
            div = DivWdg()
            div.add_class("spt_input_option")
            div.add_attr("spt_input_key", group)
            
            select = SelectWdg(my.get_name())
            select.add_empty_option("-- Select a User --")
            values = ['']
            labels = ['<< %s >>'%group]
            for user in group_dict[group]:
                values.append(user)
                labels.append('  %s'%user)
            select.add_behavior( { 'type': 'click',
               'cbjs_action': 'spt.dg_table.select_wdg_clicked( evt, bvr.src_el );' } )
            behavior = {
                'type': 'keyboard',
                'kbd_handler_name': 'DgTableSelectWidgetKeyInput',
            }
            select.add_behavior( behavior )
            select.set_option("values", values)
            select.set_option("labels", labels)
            
            div.add(select)
            top.add(div)
        
        return top

class LoginTableElementWdg(SimpleTableElementWdg):
    '''This ElementWdg is used to add the group that the given task is looking
    for in ProcessGroupSelectWdg.'''
    def handle_td(my, td):
        super(LoginTableElementWdg, my).handle_td(td)
        task = my.get_current_sobject()
        if task:
            search_type = task.get_value('search_type')
            search_id = task.get_value('search_id')
            
            if not search_type or not search_id:
                return
            
            search_key = SearchKey.build_search_key(search_type, search_id, column='id')
            
            from pyasm.common import SObjectSecurityException
            try:
                parent = Search.get_by_search_key(search_key)
                pipeline = Pipeline.get_by_sobject(parent)
               
                if pipeline:
                    attrs = pipeline.get_process_attrs(task.get_value('process'))
                
                    td.add_attr('spt_pipeline_code', attrs.get('%s_login_group'%my.get_name()))
            except SObjectSecurityException, e:
                pass
