###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['DbConfigPanelWdg', 'DbConfigWdg', 'DbConfigCbk', 'DbConfigSaveCbk']

from pyasm.common import Config, Environment, Common, TacticException
from pyasm.web import DivWdg, Table
from pyasm.widget import CheckboxWdg, TextWdg, SelectWdg, PasswordWdg
from pyasm.command import Command
from pyasm.web import WebContainer

from tactic.ui.common import BaseRefreshWdg
from tactic.ui.container import WizardWdg
from tactic.ui.widget import ActionButtonWdg
from tactic.ui.input import TextInputWdg

import os, shutil


DEFAULTS = {}
DEFAULTS['PostgreSQL'] = {
    'server': 'localhost',
    'port': None,
    'user': 'postgres',
    'password': ''
}
DEFAULTS['Oracle'] = {
    'server': 'localhost',
    'port': 1521,
    'user': '__EMPTY__',   # This should be hidden from the user?
    'password': ''
}
DEFAULTS['MySQL'] = {
    'server': 'localhost',
    'port': None,
    'user': 'root',   # This should be hidden from the user?
    'password': ''
}



class DbConfigPanelWdg(BaseRefreshWdg):
    '''Container with a nice border and centered'''

    def get_display(my):
        top = my.top

        top.set_round_corners()
        top.add_style("margin: 20px")

        top.add_border()
        top.add_color("background", "background2", +10)
        top.add_color("color", "color2")

        top.add_style("width: 430px")
        top.add_style("min-height: 500")
        top.add_style("margin-left: auto")
        top.add_style("margin-right: auto")
        top.add_style("padding: 20px")

        config_wdg = DbConfigContentWdg()
        top.add(config_wdg)

        return top



class DbConfigWdg(BaseRefreshWdg):
    '''Container with a nice border and centered'''

    def get_display(my):
        top = my.top

        top.add_border()
        top.add_color("background", "background")
        top.add_color("color", "color2")

        top.add_style("width: 430px")
        top.add_style("min-height: 500")
        top.add_style("padding: 20px")

        config_wdg = DbConfigContentWdg()
        top.add(config_wdg)

        return top


class DbConfigContentWdg(BaseRefreshWdg):

    def get_display(my):

        top = my.top
        top.add_class("spt_db_config_top")
        top.add_style("width: 430px")
        top.add_style("min-height: 500")

        title_wdg = DivWdg()
        top.add(title_wdg)
        title_wdg.add("TACTIC Configuration Setup")
        title_wdg.add_style("font-size: 20px")


        top.add("<hr/>")

        top.add("<i style='opacity: 0.5'>%s</i><br/>" % Config.get_config_path())
        top.add("<br/>")



        save_button = ActionButtonWdg(title="Start >>", tip="Save configuration and start using TACTIC")
        top.add(save_button)
        save_button.add_style("float: right")
        save_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_db_config_top");
        var values = spt.api.Utility.get_input_values(top, null, false);
        var class_name = 'tactic.ui.startup.DbConfigSaveCbk';
        var server = TacticServerStub.get();
        var kwargs = {};

        var ret_val = server.execute_cmd(class_name, kwargs, values);
        var info = ret_val.info;

        // This means TACTIC was restarted
        if (typeof(info) == 'undefined') {
            spt.app_busy.show("Restarting TACTIC ...");
            setTimeout( function() {
                window.location = '/tactic';
            }, 2000 );
        }
        else if (info.error) {
            spt.alert(info.error);
        }
        else {
            window.location = '/tactic';
        }
        '''
        } )


        # DISABLING for 3.7
        """
        vendor = Config.get_value("database", "vendor")

        title_wdg = DivWdg()
        top.add(title_wdg)
        title_wdg.add("<b>Database Setup</b>")
        title_wdg.add_style("margin-bottom: 10px")


        db_select = SelectWdg("database/vendor")
        db_select.set_option("labels", "SQLite|PostgreSQL|MySQL|Oracle|SQLServer")
        db_select.set_option("values", "Sqlite|PostgreSQL|MySQL|Oracle|SQLServer")

        db_select.set_value(vendor)



        db_select.add_behavior( {
        'type': 'change',
        'cbjs_action': '''

        var key;
        if (bvr.src_el.value == 'Sqlite') {
            key = 'Sqlite';
        }
        else {
            key = 'Other';
        }

        var top = bvr.src_el.getParent(".spt_db_config_top");
        var options_els = top.getElements(".spt_db_options");
        for (var i = 0; i < options_els.length; i++) {
            var vendor = options_els[i].getAttribute("spt_vendor");
            if (vendor == key) {
                spt.show(options_els[i]);
            }
            else {
                spt.hide(options_els[i]);
            }

        }
        '''
        } )
 
        option_div = DivWdg()
        top.add(option_div)
        option_div.add("Vendor: ")
        option_div.add(db_select)
        option_div.add_style("margin: 20px")

        sqlite_wdg = my.get_sqlite_wdg()
        option_div.add(sqlite_wdg)
        otherdb_wdg = my.get_otherdb_wdg()
        option_div.add(otherdb_wdg)

        if vendor == 'Sqlite':
            otherdb_wdg.add_style("display: none")
            sqlite_wdg.add_style("display", "")
        else:
            otherdb_wdg.add_style("display", "")
            sqlite_wdg.add_style("display: none")


        test_button = ActionButtonWdg(title="<< Test >>", tip="Test connecting to database")
        option_div.add(test_button)
        test_button.add_style("margin-left: auto")
        test_button.add_style("margin-right: auto")
        test_button.add_behavior( {
        'type': 'click_up',
        'cbjs_action': '''
        var top = bvr.src_el.getParent(".spt_db_config_top");
        var values = spt.api.Utility.get_input_values(top, null, false);
        var class_name = 'tactic.ui.startup.DbConfigCbk';
        var server = TacticServerStub.get();
        var kwargs = {};
        var ret_val = server.execute_cmd(class_name, kwargs, values);
        var info = ret_val.info;
        if (info.error) {
            spt.error(info.error);
        }
        else {
            spt.info("Connection to database successful");
        }
        '''
        } )


        top.add("<hr/>")
        """

        top.add("<br/>")
        top.add("<br/>")

        title = "Asset Management Setup"
        category = "checkin"

        if os.name == "nt":
            options = ['asset_base_dir', 'win32_server_handoff_dir', 'win32_client_handoff_dir','linux_client_handoff_dir']
        else:
            options = ['asset_base_dir', 'linux_server_handoff_dir', 'win32_client_handoff_dir','linux_client_handoff_dir']

        top.add( my.configure_category(title, category, options) )



        top.add("<hr/>")

        title = "Mail Server"
        category = "services"
        options = ['mailserver', 'mail_user', 'mail_password', 'mail_port', 'mail_tls_enabled']
        top.add( my.configure_category(title, category, options) )







        top.add("<hr/>")

        title = "Look and Feel"
        category = "look"
        options = ['palette']
        top.add( my.configure_category(title, category, options) )







        #wizard_wdg = WizardWdg()
        #top.add(wizard_wdg)
        #wizard_wdg.add(DivWdg("cow"), "cow")
        #wizard_wdg.add(DivWdg("pig"), "pig")
        #wizard_wdg.add(DivWdg("dog"), "dog")

        return top



    def configure_category(my, title, category, options):
        div = DivWdg()

        title_wdg = DivWdg()
        div.add(title_wdg)

        #from tactic.ui.widget.swap_display_wdg import SwapDisplayWdg
        #swap = SwapDisplayWdg()
        #div.add(swap)

        title_wdg.add("<b>%s</b>" % title)


        table = Table()
        div.add(table)
        #table.add_color("color", "color")
        table.add_style("color: #000")
        table.add_style("margin: 20px")

        for option in options:
            table.add_row()
            display_title = Common.get_display_title(option)
            td = table.add_cell("%s: " % display_title)
            td.add_style("width: 150px")


            text = TextInputWdg(name="%s/%s" % (category, option))

            value = Config.get_value(category, option)
            if value:
                text.set_value(value)

            table.add_cell(text)

        return div




    def get_sqlite_wdg(my):
        div = DivWdg()
        div.add_class("spt_db_options")
        div.add_attr("spt_vendor", "Sqlite")
        div.add_style("padding: 20px")

        div.add("Database Folder: ")
        text = TextInputWdg(name="database/sqlite_db_dir")
        div.add(text)


        db_dir = Config.get_value("database", "sqlite_db_dir")
        if not db_dir:
            data_dir = Environment.get_data_dir()
            db_dir = "%s/db" % data_dir

        text.set_value(db_dir)

        return div


    def get_otherdb_wdg(my):

        div = DivWdg()
        div.add_class("spt_db_options")
        div.add_attr("spt_vendor", "Other")
        div.add_style("margin: 20px")

        table = Table()
        div.add(table)
        table.add_color("color", "color")


        table.add_row()
        table.add_cell("Server: ")
        text = TextInputWdg(name="server")
        text.set_value("localhost")
        table.add_cell(text)
        server = Config.get_value("database", "server")
        if server:
            text.set_value(server)



        table.add_row()
        table.add_cell("Port: ")
        text = TextInputWdg(name="port")
        table.add_cell(text)
        port = Config.get_value("database", "port")
        if port:
            text.set_value(port)




        table.add_row()
        table.add_cell("Login: ")
        text = TextInputWdg(name="user")
        table.add_cell(text)
        user = Config.get_value("database", "user")
        if user:
            text.set_value(user)


        table.add_row()
        text = PasswordWdg("password")
        table.add_cell("Password: ")
        table.add_cell(text)
        password = Config.get_value("database", "password")
        if password:
            text.set_value(password)


        #from pyasm.search import Sql
        #sql.connect()



        return div



    def get_asset_dir_wdg(my):
        div = DivWdg()

        return div




class DbConfigCbk(Command):

    def execute(my):

        # make sure tmp config is unset.
        Config.unset_tmp_config()
        Config.reload_config()

        web = WebContainer.get_web()

        vendor = web.get_form_value("database/vendor")


        if vendor == 'Sqlite':
            db_dir = web.get_form_value("sqlite_db_dir")
            database = "sthpw"
            db_path = "%s/%s.db" % (db_dir, database)

            if os.path.exists(db_path):
                return

        elif vendor == 'PostgreSQL':
            my.test_postgres(vendor)
            return

        elif vendor == 'MySQL':
            my.test_postgres(vendor)
            return

        my.info['error'] = "Cannot connect to database"

    def check_database_schema(my):
        pass


    def test_postgres(my, vendor):
        web = WebContainer.get_web()

        defaults = DEFAULTS[vendor]

        default_server = defaults['server']
        default_port = defaults['port']
        default_user = defaults['user']
        default_password = defaults['password']

        server = web.get_form_value("server")
        if not server:
            server = default_server
        port = web.get_form_value("port")
        if not port:
            port = default_port
        else:
            port = int(port)
        user = web.get_form_value("user")
        if not user:
            user = default_user
        password = web.get_form_value("password")
        if not password:
            password = default_password



        from pyasm.search import Sql
        sql = Sql("sthpw", server, user, password=password, vendor=vendor, port=port)
        try:
            sql.connect()
            print sql.do_query("select id from transaction_log")
        except Exception, e:
            my.info['error'] = "Could not connect to database with (vendor=%s, server=%s, user=%s, port=%s)" % (vendor, server, user, port)
            my.info['message'] = str(e)
            print e



class DbConfigSaveCbk(Command):

    def execute(my):


        # make sure tmp config is unset.
        Config.unset_tmp_config()
        Config.reload_config()

        web = WebContainer.get_web()

        # read the current config file.


        # copy config to the path
        config_path = Config.get_config_path()
        if not os.path.exists(config_path):
            print "Installing default config file"

            dirname = os.path.dirname(config_path)
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            if os.name == 'nt':
                osname = 'win32'
            else:
                osname = 'linux'

            install_dir = Environment.get_install_dir()
            install_config_path = "%s/src/install/start/config/tactic_%s-conf.xml" % ( install_dir, osname)

            shutil.copy(install_config_path, dirname)

        #my.configure_db()
        my.configure_services()
        my.configure_asset_dir()
        my.configure_palette()
        my.save_config()


        # close all of the database connections
        """
        from pyasm.search import DbContainer
        DbContainer.close_thread_sql()

        from pyasm.common import Container
        Container.clear_all()

        from pyasm.security import Batch
        Batch()
        """


        # remove the first run file
        data_dir = Environment.get_data_dir()
        path = "%s/first_run" % data_dir
        if os.path.exists(path):
            os.unlink(path)


        my.restart_program()


    def restart_program(my):
        '''Restarts the current program.'''
        import sys
        python = sys.executable
        os.execl(python, python, * sys.argv)





    def configure_db(my):

        web = WebContainer.get_web()

        vendor = web.get_form_value("database/vendor")

        if vendor == 'Sqlite':
            # take the current files and copy them to the database folder
            db_dir = web.get_form_value("database/sqlite_db_dir")
            if not db_dir:
                raise TacticException("No Folder configured for Sqlite Database")

            if not os.path.exists(db_dir):
                os.makedirs(db_dir)

            # check to see if the sthpw database is in this folder
            sthpw_path = "%s/sthpw.db" % (db_dir)
            if not os.path.exists(sthpw_path):
                # copy the default database over
                install_dir = Environment.get_install_dir()
                template_db = "%s/src/install/start/db/sthpw.db" % install_dir
                shutil.copy(template_db, db_dir)

            Config.set_value("database", "sqlite_db_dir", db_dir)

            Config.remove("database", "server")
            Config.remove("database", "port")
            Config.remove("database", "user")
            Config.remove("database", "password")


        else:
            defaults = DEFAULTS[vendor]

            server = defaults['server']
            port = defaults['port']
            user = defaults['user']
            password = defaults['password']

            Config.remove("database", "sqlite_db_dir")

            if server:
                Config.set_value("database", "server", server)
            else:
                Config.remove("database", "server")

            if port:
                Config.set_value("database", "port", port)
            else:
                Config.remove("database", "port")

            if user:
                Config.set_value("database", "user", user)
            else:
                Config.remove("database", "user")

            if password:
                Config.set_value("database", "password", password)
            else:
                Config.remove("database", "password")

        # save the database
        Config.set_value("database", "vendor", vendor)



    def configure_asset_dir(my):

        web = WebContainer.get_web()
        asset_dir = web.get_form_value("checkin/asset_base_dir")

        if asset_dir != None:
            if asset_dir and not os.path.exists(asset_dir):
                os.makedirs(asset_dir)
            Config.set_value("checkin", "asset_base_dir", asset_dir)


        linux_server_handoff = web.get_form_value("checkin/linux_server_handoff_dir")
        print "linux: ", linux_server_handoff
        if linux_server_handoff:
            Config.set_value("checkin", "linux_server_handoff_dir", linux_server_handoff)
        else:
            Config.remove("checkin", "linux_server_handoff_dir")


        win32_client_handoff = web.get_form_value("checkin/win32_client_handoff_dir")
        if win32_client_handoff != None:
            Config.set_value("checkin", "win32_client_handoff_dir", win32_client_handoff)
        else:
            Config.remove("checkin", "win32_client_handoff_dir")


    def configure_palette(my):

        web = WebContainer.get_web()
        palette = web.get_form_value("look/palette")

        if palette:
            Config.set_value("look", "palette", palette)
        else:
            Config.remove("look", "palette")


    def configure_services(my):

        web = WebContainer.get_web()

        options = ['server', '_user', '_password', '_port', '_tls_enabled']
        for option in options:
            server = web.get_form_value("services/mail%s" %option)

            if server:
                Config.set_value("services", "mail%s" %option, server)
            else:
                Config.remove("services", "mail%s"%option)




    def save_config(my):
        Config.save_config()
        Config.reload_config()



